// -*- C++ -*-
/*!
 * @file  Camera.cpp
 * @brief Console input component
 * $Date$
 *
 * $Id$
 */

#include "Camera.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>
#include <memory.h>
#include <math.h>

#include "shm.h"

#define MAX_IMAGESIZE	1024	/* $BA[Dj$9$k=D!&2#$N:GBg2hAG?t(B */
#define MAX_BRIGHTNESS  255		/* $BA[Dj$9$k:GBg3,D4CM(B */
#define MAX_BUFFERSIZE  256		/* $BMxMQ$9$k%P%C%U%!:GBgD9(B */
#define RGB               3

unsigned char image_in_L[3][240][320];
unsigned char image_in_R[3][240][320];
short i, j, k;
int x_size = 0, y_size = 0;

void load_ppm_data(char *file_name, unsigned char image[3][240][320], int *x_size, int *y_size);

int shm_initialize(key_t shm_key, int shm_size,int *shrd_id,int *shm_ptr);
int shm_access(key_t shm_key, int shm_size, int *shrd_id, int *shm_ptr);

int shd_param_id;
struct SHM_PARAM *param_ptr;
double *data_ptr;
key_t key;

// Module specification
// <rtc-template block="module_spec">
static const char* camera_spec[] =
  {
    "implementation_id", "Camera",
    "type_name",         "Camera",
    "description",       "Console input component",
    "version",           "1.0",
    "vendor",            "MyName",
    "category",          "example",
    "activity_type",     "DataFlowComponent",
    "max_instance",      "10",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables

    ""
  };
// </rtc-template>

Camera::Camera(RTC::Manager* manager)
  : RTC::DataFlowComponentBase(manager),
    // <rtc-template block="initializer">
    Cam_inIn("Cam_in", Cam_in),
    Cam_outOut("Cam_out", Cam_out),
    
    // </rtc-template>
	dummy(0)
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  registerInPort("Cam_in", Cam_inIn);
  
  // Set OutPort buffer
  registerOutPort("Cam_out", Cam_outOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

}

Camera::~Camera()
{
}


RTC::ReturnCode_t Camera::onInitialize()
{
  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  
	// sharedmemory$B$NNN0h3NJ](B
	system("./shmexe &");
	sleep(1);

	//	$B%+%a%i5/F0(B
	system("./capture_imageprocess &");

	key = ftok(KEYFILE_PATH,PROJ_CHAR);
  if( key == -1 ) {
    perror("ftok()");
  }

  shm_access(key, sizeof(struct SHM_PARAM), &shd_param_id, (int *)		&param_ptr);
  param_ptr->shd_param_id = shd_param_id;

  // </rtc-template>
  return RTC::RTC_OK;
}



/*
RTC::ReturnCode_t Camera::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Camera::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Camera::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Camera::onActivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Camera::onDeactivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t Camera::onExecute(RTC::UniqueId ec_id)
{
	if (Cam_inIn.isNew())
	{
		Cam_inIn.read();
		if (Cam_in.data == 1)
		{
			//	$B2hA|<hF@;X<((B
			param_ptr->capture_flag = 1;

			//	$B2hA|J]B8BT$A(B($BL5$/$F$b%$%$$+$b!D!K(B
			while(param_ptr->capture_flag == 1){}

			load_ppm_data("./picture1.ppm", image_in_L, &x_size, &y_size);
			load_ppm_data("./picture2.ppm", image_in_R, &x_size, &y_size);

			for(i=0;i<3;i++){
				for(j=0;j<240;j++){
					for(k=0;k<320;k++){
          Cam_out.data.gazou_L[i][j][k] = image_in_L[i][j][k];
          Cam_out.data.gazou_R[i][j][k] = image_in_R[i][j][k]; 
					}
				}
			}

			Cam_outOut.write();
		}
	}

	return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t Camera::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Camera::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Camera::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Camera::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Camera::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void CameraInit(RTC::Manager* manager)
  {
    RTC::Properties profile(camera_spec);
    manager->registerFactory(profile,
                             RTC::Create<Camera>,
                             RTC::Delete<Camera>);
  }
  
};

void load_ppm_data(char *file_name,unsigned char image[3][240][320],int *x_size, int *y_size)
{
	char buffer[256];  /* $B%G!<%?FI$_9~$_MQ:n6HJQ?t(B */
	FILE *fp;					 /* $B%U%!%$%k%]%$%s%?(B */
	int max_bright;		 /* $B:GBg3,D4CM(B */
	int x, y; 						/* $B%k!<%WJQ?t(B */

	/* $BF~NO%U%!%$%k$N%*!<%W%s(B */
	fp = fopen( file_name, "rb" );

	/* $B%U%!%$%k%?%$%W(B(=P6)$B$N3NG'(B */
	fgets( buffer, MAX_BUFFERSIZE, fp );
//	if ( buffer[0] != 'P' || buffer[1] != '6' ){
//       printf("%c, %c\n", buffer[0], buffer[1]);
//	printf("$B%U%!%$%k$N%U%)!<%^%C%H$,(B P6 $B$H$O0[$J$j$^$9!%(B\n");
//		exit(1);
//	}
	/* x_size, y_size $B$NBeF~!J(B#$B$+$i;O$^$k%3%a%s%H$OFI$_Ht$P$9!K(B */
	*x_size = 0;
	*y_size = 0;

	while ( *x_size == 0 || *y_size == 0 ){
		fgets( buffer, MAX_BUFFERSIZE, fp );
		if ( buffer[0] != '#' ){
			sscanf( buffer, "%d %d", x_size, y_size );
		}
	}
	/* max_bright $B$NBeF~!J(B#$B$+$i;O$^$k%3%a%s%H$OFI$_Ht$P$9!K(B */
	max_bright = 0;
	while ( max_bright == 0 ){
		fgets( buffer, MAX_BUFFERSIZE, fp );
		if ( buffer[0] != '#' ){
			sscanf( buffer, "%d", &max_bright );
		}
	}
	/* $B%Q%i%a!<%?$N2hLL$X$NI=<((B */
	if ( *x_size > MAX_IMAGESIZE || *y_size > MAX_IMAGESIZE ){
		cout << "Range Over!" << endl;
		//printf("$BA[DjCM(B %d x %d $B$rD6$($F$$$^$9!%(B\n", MAX_IMAGESIZE, MAX_IMAGESIZE);
		//printf("$B$b$&>/$7>.$5$J2hA|$r;H$C$F2<$5$$!%(B\n");
		exit(1);
	}
	if ( max_bright != 255 ){
		cout << "Wrong Max Gradation!" << endl;
		//printf("$B:GBg3,D4CM$,ITE,@Z$G$9!%(B\n");
		exit(1);
	}

	/* $B2hA|%G!<%?$rFI$_9~$s$G2hA|MQG[Ns$KBeF~$9$k(B */
	for ( y = 0; y < 240/**y_size*/; y ++ ){
		for ( x = 0; x < 320/**x_size*/; x ++ ){
			image[0][y][x] = (unsigned char)fgetc( fp );
			image[1][y][x] = (unsigned char)fgetc( fp );
			image[2][y][x] = (unsigned char)fgetc( fp );
		}
	}

	fclose(fp);
}


