#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>
#include <memory.h>
#include <math.h>
#include "params.h"

#define R_Num             0		/* H 色情報 */
#define G_Num             1		/* S 色情報 */
#define B_Num             2		/* V 色情報 */
#define MAX_COLOR_RENG	255		/* 最大カラー範囲 */
#define MIN_COLOR_RENG    0		/* 最小カラー範囲 */
#define RGB               3		/* 色情報 */
#define MAX_IMAGESIZE	1024	/* 想定する縦・横の最大画素数 */
#define MAX_BRIGHTNESS  255		/* 想定する最大階調値 */
#define GRAYLEVEL       256		/* 想定する階調数(=最大階調値+1) */
#define MAX_FILENAME    256		/* 想定するファイル名の最大長 */
#define MAX_BUFFERSIZE  256		/* 利用するバッファ最大長 */
#define COLOR			0		/* RGBのカラーを選択 */
#define IMAGESIZE 		256   	/* = GRAYLEVEL */
#define INPUT_GRAY_SIZE	4		/* ファイルから読み込む際のバッファサイズ (要適宜変更)*/

#define FILE_OPEN_ERROR(ee){if(ee == NULL){printf("FILE_OPEN_ERROR : at %s %d¥n",__FILE__,__LINE__);exit(1);}else fp = ee;} // ファイルオープンエラー検知


unsigned char	image_in[RGB][MAX_IMAGESIZE][MAX_IMAGESIZE];  // 入力画像用配列（[RGB]には色情報が入る）
unsigned char	image_out[RGB][MAX_IMAGESIZE][MAX_IMAGESIZE]; // 出力画像用配列

unsigned char      image_in_R[MAX_IMAGESIZE][MAX_IMAGESIZE];    //入出力画像用配列（RGB２次元用）
unsigned char      image_in_G[MAX_IMAGESIZE][MAX_IMAGESIZE];    
unsigned char      image_in_B[MAX_IMAGESIZE][MAX_IMAGESIZE];    
unsigned char      image_out_R[MAX_IMAGESIZE][MAX_IMAGESIZE];    
unsigned char      image_out_G[MAX_IMAGESIZE][MAX_IMAGESIZE];    
unsigned char      image_out_B[MAX_IMAGESIZE][MAX_IMAGESIZE];   
 
unsigned char image_pgm_in[MAX_IMAGESIZE][MAX_IMAGESIZE];  //pgm画像用データ格納配列
unsigned char image_pgm_out[MAX_IMAGESIZE][MAX_IMAGESIZE];


// 関数のプロトタイプ宣言
void load_ppm_data(char *file_name, unsigned char image[RGB][MAX_IMAGESIZE][MAX_IMAGESIZE], int *x_size, int *y_size);	/* 画像読み込み用関数 */

//void load_pgm_data(char *file_name, unsigned char image[RGB][MAX_IMAGESIZE][MAX_IMAGESIZE], int *x_size, int *y_size );	/* 画像読み込み用関数 */
void save_ppm_data(char *file_name, unsigned char image[RGB][MAX_IMAGESIZE][MAX_IMAGESIZE], int x_size, int y_size);	/* 画像の出力用関数 */
//void save_pgm_data(char *file_name, unsigned char image[MAX_IMAGESIZE][MAX_IMAGESIZE], int x_size, int y_size);			/* pgm ファイルの作成 */
void out_text_file(char *file_name, unsigned char image[RGB][MAX_IMAGESIZE][MAX_IMAGESIZE], int x_size, int y_size);			/* 画像情報をファイルに書き込む */
void trans3to2(unsigned char imageR[MAX_IMAGESIZE][MAX_IMAGESIZE],unsigned char imageG[MAX_IMAGESIZE][MAX_IMAGESIZE],unsigned char imageB[MAX_IMAGESIZE][MAX_IMAGESIZE],unsigned char image[RGB][MAX_IMAGESIZE][MAX_IMAGESIZE],int x_size, int y_size); /*画像配列変換関数(RGB３次元→２次元)*/
void trans2to3(unsigned char imageR[MAX_IMAGESIZE][MAX_IMAGESIZE],unsigned char imageG[MAX_IMAGESIZE][MAX_IMAGESIZE],unsigned char imageB[MAX_IMAGESIZE][MAX_IMAGESIZE],unsigned char image[RGB][MAX_IMAGESIZE][MAX_IMAGESIZE],int x_size, int y_size); /*画像配列変換関数（RGB２次元→３次元）*/
void histgram(unsigned char image_in[MAX_IMAGESIZE][MAX_IMAGESIZE], long hist[256],int x_size,int y_size);/*ヒストグラムを求める関数*/
void histprint(long hist[256], char *buf,int x_size,int y_size);/*ヒストグラムを表示する関数*/
void thresh_color(unsigned char image_in_r[MAX_IMAGESIZE][MAX_IMAGESIZE],
		  unsigned char image_in_g[MAX_IMAGESIZE][MAX_IMAGESIZE],
		  unsigned char image_in_b[MAX_IMAGESIZE][MAX_IMAGESIZE],
		  unsigned char image_out_r[MAX_IMAGESIZE][MAX_IMAGESIZE],
		  unsigned char image_out_g[MAX_IMAGESIZE][MAX_IMAGESIZE],
		  unsigned char image_out_b[MAX_IMAGESIZE][MAX_IMAGESIZE],
		  int thdrl, int thdrm, int thdgl, int thdgm, int thdbl, int thdbm);/*RGBによる閾値処理*/

void gradient(unsigned char image_in[MAX_IMAGESIZE][MAX_IMAGESIZE],
	      unsigned char image_out[MAX_IMAGESIZE][MAX_IMAGESIZE], float amp); /*１次微分による輪郭抽出*/

void threshold(unsigned char image_in[MAX_IMAGESIZE][MAX_IMAGESIZE],unsigned char image_out[MAX_IMAGESIZE][MAX_IMAGESIZE],int thres);

void load_pgm_data(char *file_name,unsigned char image[MAX_IMAGESIZE][MAX_IMAGESIZE],int *x_size, int *y_size );  /*pgmデータを読み込む*/

void save_pgm_data(char *file_name,unsigned char image[MAX_IMAGESIZE][MAX_IMAGESIZE],int x_size, int y_size); /*pgmデータを保存*/

void laplacian_color(unsigned char image_in[3][MAX_IMAGESIZE][MAX_IMAGESIZE],
	       unsigned char image_out[3][MAX_IMAGESIZE][MAX_IMAGESIZE], float amp);/*カラー画像の輪郭抽出(輪郭を白線で抽出)*/

//void capturing(void/*int argc, char *argv[]*/); /*カメラ画像キャプチャ関数*/
int x_size1, y_size1, /* image1 の横画素数，縦画素数 */
    x_size2, y_size2; /* image2 の横画素数，縦画素数 */

int image_process(int num)
{
	int x,y;
      long i;
      long hist_R[256];  //ヒストグラム用の配列（RGB３色）
      long hist_G[256];
      long hist_B[256];
      long hist_pgm[256];
      char hist_buf_R[100000]; //ヒストグラム表示用配列（RGB３色）
      char hist_buf_G[100000];
      char hist_buf_B[100000];
      char hist_buf_pgm[100000];
     
       char *pic_num=NULL;

       pic_num = calloc(80,1);

      snprintf(pic_num, 30,"./picture%d",num);
       printf("%s\n",pic_num);
     puts("=====image processing=====");  //画像処理開始

       
     load_ppm_data(pic_num,image_in,&x_size1,&y_size1); 
                                  // 処理用PPM画像のインポート
      printf("check\n");
      /*for(x=0;x<320;x++){
            for(y=0;y<240;y++){
                   image_out[0][y][x]=image_in[0][y][x];
                        image_out[1][y][x]=image_in[1][y][x];
                       image_out[2][y][x]=image_in[2][y][x];
              }
      }*/
     printf("check\n");
     trans3to2(image_in_R,image_in_G,image_in_B,image_in,x_size1,y_size1); 
                    //３次元配列を２次元配列に変換


       //thresh_color(image_in_R,image_in_G,image_in_B,image_out_R,image_out_G,image_out_B,62,72,65,82,45,57);

      gradient(image_in_R,image_out_R,5);
      gradient(image_in_G,image_out_G,5);
      gradient(image_in_B,image_out_B,5); 
      
    /* histgram(image_in_R,hist_R,x_size1,y_size1); //ヒストグラムを求める。
     histgram(image_in_G,hist_G,x_size1,y_size1);
     histgram(image_in_B,hist_B,x_size1,y_size1);
*/
  /*   histprint(hist_R,hist_buf_R,x_size1,y_size1); //ヒストグラムを表示するための配列を作成
     histprint(hist_G,hist_buf_G,x_size1,y_size1);
     histprint(hist_B,hist_buf_B,x_size1,y_size1);
     */ 
	//printf("hist_R\n%s",hist_buf_R);  //ヒストグラムを表示する。
      //printf("hist_G\n%s",hist_buf_G);
     // printf("hist_B\n%s",hist_buf_B);

      //threshold(image_out_R,image_out_R,100);
      //threshold(image_out_G,image_out_G,100);
      //threshold(image_out_B,image_out_B,100);
      /*thresh_color(image_in_R,image_in_G,image_in_B,image_out_R,image_out_G,image_out_B,72,90,126,153,98,114);
*/

     trans2to3 (image_out_R,image_out_G,image_out_B,image_out,x_size1,y_size1);
              //２次元配列を３次元配列に戻す


      //laplacian_color(image_in,image_out,5);
     save_ppm_data("./output.ppm", image_out, x_size1, y_size1); //画像処理後の画像を出力
	//out_text_file("/home/taro/Desktop/image_processing/output.txt", image_in, x_size1, y_size1);			// 処理用PPM画像の情報をデータとして保存

    /* load_pgm_data("/home/taro/Desktop/picture.pgm",image_pgm_in,&x_size1,&y_size1);

	
       gradient(image_pgm_in,image_pgm_out,5);
       histgram(image_pgm_out,hist_pgm,x_size1,x_size1);
       histprint(hist_pgm,hist_buf_pgm,x_size1,y_size1);
        printf("%s",hist_buf_pgm);
       save_pgm_data("/home/taro/Desktop/output2.pgm",image_pgm_out,x_size1,y_size1);*/
  //capturing(2);
  //capturing(3);

}
/*=============================================================================
	関数名  load_ppm_data
	用途    PPM画像の入力
	引数    char *file_name			: 入力画像ファイル名のポインタ
			unsigned char image_		: 入力画像格納配列のポインタ
			int x_size					: 画像横サイズのポインタ
			int y_size					: 画像横サイズのポインタ
	返り値  non
=============================================================================*/
void load_ppm_data(char *file_name,unsigned char image[RGB][MAX_IMAGESIZE][MAX_IMAGESIZE],int *x_size, int *y_size)
{
	char buffer[MAX_BUFFERSIZE];  /* データ読み込み用作業変数 */
	FILE *fp; /* ファイルポインタ */
	int max_bright; /* 最大階調値 */
	int x, y; /* ループ変数 */
	/* 入力ファイルのオープン */
	FILE_OPEN_ERROR(fopen( file_name, "rb" ));

	/* ファイルタイプ(=P6)の確認 */
	fgets( buffer, MAX_BUFFERSIZE, fp );
	if ( buffer[0] != 'P' || buffer[1] != '6' ){
	printf("ファイルのフォーマットが P6 とは異なります．\n");
		exit(1);
	}
	/* x_size, y_size の代入（#から始まるコメントは読み飛ばす） */
	*x_size = 0;
	*y_size = 0;

	while ( *x_size == 0 || *y_size == 0 ){
		fgets( buffer, MAX_BUFFERSIZE, fp );
		if ( buffer[0] != '#' ){
			sscanf( buffer, "%d %d", x_size, y_size );
		}
	}
	/* max_bright の代入（#から始まるコメントは読み飛ばす） */
	max_bright = 0;
	while ( max_bright == 0 ){
		fgets( buffer, MAX_BUFFERSIZE, fp );
		if ( buffer[0] != '#' ){
			sscanf( buffer, "%d", &max_bright );
		}
	}
	/* パラメータの画面への表示 */
	if ( *x_size > MAX_IMAGESIZE || *y_size > MAX_IMAGESIZE ){
		printf("想定値 %d x %d を超えています．\n", MAX_IMAGESIZE, MAX_IMAGESIZE);
		printf("もう少し小さな画像を使って下さい．\n");
		exit(1);
	}
	if ( max_bright != MAX_BRIGHTNESS ){
		printf("最大階調値が不適切です．\n");
		exit(1);
	}

	/* 画像データを読み込んで画像用配列に代入する */
	for ( y = 0; y < 240/**y_size*/; y ++ ){
		for ( x = 0; x < 320/**x_size*/; x ++ ){
			image[0][y][x] = (unsigned char)fgetc( fp );
			image[1][y][x] = (unsigned char)fgetc( fp );
			image[2][y][x] = (unsigned char)fgetc( fp );
		}
	}

	fclose(fp);
}

/*=============================================================================
	関数名  save_ppm_data
	用途    PPM画像の保存
	引数    char *file_name				: 出力画像ファイル名のポインタ
			unsigned char image_		: 出力画像格納配列のポインタ
			int x_size					: 画像横サイズ
			int y_size					: 画像横サイズ
	返り値  non
=============================================================================*/
void save_ppm_data(char *file_name,unsigned char image[RGB][MAX_IMAGESIZE][MAX_IMAGESIZE],int x_size, int y_size)
/* image2[ ][ ], x_size2, y_size2 のデータを，それぞれ ppm 画像，*/
/* 横画素数，縦画素数としてファイルに保存する．                  */
{
    FILE *fp; /* ファイルポインタ */
    int x, y, k; /* ループ変数 */
    /* 出力ファイルのオープン */
    FILE_OPEN_ERROR(fopen(file_name, "wb"));
	
    /* ファイル識別子 "P6" を先頭に出力する */
    fputs( "P6\n", fp );
    /* # で始まるコメント行（省略可能） */
    //fputs( "# Created by Image Processing\n", fp );

    /* 画像の横幅，縦幅の出力 */
    fprintf( fp, "%d %d\n", x_size, y_size );

    /* 最大階調値の出力 */
    fprintf( fp, "%d\n", MAX_BRIGHTNESS );

    /* 画像データの出力 */
    for ( y = 0; y < 240/*y_size*/; y ++ ){
        for ( x = 0; x <320/* x_size*/; x ++ ){
		 	for ( k = 0; k < RGB; k ++ ){
            	fputc( image[k][y][x], fp );
                        }
          }
     }
 
fclose(fp);

}


/*=============================================================================
	関数名  out_text_file
	用途    入力画像データのファイル出力
	引数    char *file_name				: 保存ファイル名のポインタ
			unsigned char image_		: 保存画像格納配列のポインタ
			int x_size					: 画像横サイズ格納配列のポインタ
			int y_size					: 画像横サイズ格納配列のポインタ
	返り値  non
=============================================================================*/
void out_text_file(char *file_name,unsigned char image[RGB][MAX_IMAGESIZE][MAX_IMAGESIZE],int x_size, int y_size)
{
	// 変数の定義
	int i,j;  // ループカウント計測用
	FILE *fp; // ファイルポインタ
	// 入力ファイルのオープン
	FILE_OPEN_ERROR(fopen( file_name, "w+" ));
	for(i=0;i<y_size;i++)
		for(j=0;j<x_size;j++)
			fprintf(fp,"%d¥t%d¥t%d¥t%d¥t%d¥n",i,j,image[0][i][j],image[1][i][j],image[2][i][j]);
	fclose(fp);
}


/*=============================================================================
	関数名  trans3to2（2007/6/29:鈴木太郎）
	用途    取得したPPM画像のデータの３次元配列を２次元配列に変換
	引数   	unsigned char imageR[MAX_IMAGESIZE][MAX_IMAGESIZE]:2次元に変換後の格納配列（赤）
unsigned char imageG[MAX_IMAGESIZE][MAX_IMAGESIZE]:2次元に変換後の格納配列（緑）
unsigned char imageB[MAX_IMAGESIZE][MAX_IMAGESIZE]:2次元に変換後の格納配列（青）unsigned char image[RGB][MAX_IMAGESIZE][MAX_IMAGESIZE]:変換前の３次元配列格納配列int x_size, int y_size	: 入力画像格納配列のポインタ
			int x_size					: 画像横サイズ
			int y_size					: 画像縦サイズ
	返り値  non
=============================================================================*/
void trans3to2(unsigned char imageR[MAX_IMAGESIZE][MAX_IMAGESIZE],unsigned char imageG[MAX_IMAGESIZE][MAX_IMAGESIZE],unsigned char imageB[MAX_IMAGESIZE][MAX_IMAGESIZE],unsigned char image[RGB][MAX_IMAGESIZE][MAX_IMAGESIZE],int x_size, int y_size)
{
         // 変数の定義
	int i,j;  // ループカウント計測用
        
        //3次元配列→２次元配列に変換
       for(i=0;i<y_size;i++){
          for(j=0;j<x_size;j++){
                 imageR[i][j]=image[0][i][j];
                 imageG[i][j]=image[1][i][j];
                 imageB[i][j]=image[2][i][j];
            }
       }
}

/*=============================================================================
	関数名  trans2to3(2007/6/29:鈴木太郎)
	用途    ２次元配列に変換した３次元配列のPPM画像データを３次元配列に戻す
	引数   unsigned char imageR[MAX_IMAGESIZE][MAX_IMAGESIZE]:変換前の格納配列（赤）
unsigned char imageG[MAX_IMAGESIZE][MAX_IMAGESIZE]:変換前の格納配列（緑）
unsigned char imageB[MAX_IMAGESIZE][MAX_IMAGESIZE]:変換前の格納配列（青）unsigned char image[RGB][MAX_IMAGESIZE][MAX_IMAGESIZE]:３次元に変換後の格納配列
			int x_size					: 画像横サイズ
			int y_size					: 画像縦サイズ
	返り値  non
=============================================================================*/
void trans2to3(unsigned char imageR[MAX_IMAGESIZE][MAX_IMAGESIZE],unsigned char imageG[MAX_IMAGESIZE][MAX_IMAGESIZE],unsigned char imageB[MAX_IMAGESIZE][MAX_IMAGESIZE],unsigned char image[RGB][MAX_IMAGESIZE][MAX_IMAGESIZE],int x_size, int y_size)
{
         // 変数の定義
	int i,j;  // ループカウント計測用
        
        //3次元配列→２次元配列に変換
      for(i=0;i<y_size;i++){
          for(j=0;j<x_size;j++){
                 image[0][i][j]=imageR[i][j];
                 image[1][i][j]=imageG[i][j];
                 image[2][i][j]=imageB[i][j];
            }
       }
}



/*===========================================================
関数名：histgram 
用途: ヒストグラムを求める処理 
引数：  image_in:  入力画像配列
  　　　hist:      ヒストグラム用配列
      x_size:画像の横サイズ
　　　  y_size:画像の縦サイズ
=============================================================*/

void histgram(unsigned char image_in[MAX_IMAGESIZE][MAX_IMAGESIZE], long hist[256],int x_size,int y_size)
{
  int i,j,n;

  for(n=0;n<256;n++)hist[n] = 0;
  for(i=0;i<y_size;i++){
    for(j=0;j<x_size;j++){
      n = image_in[i][j];
      hist[n]++;
    }
  }
}


/*===========================================================
関数名：histprint 
用途: ヒストグラムを表示する関数 
引数：  hist:ヒストグラム配列
        char:記憶用バッファ
      x_size:画像の横サイズ
　　　  y_size:画像の縦サイズ
=============================================================*/


void histprint(long hist[256], char *buf,int x_size,int y_size)
{
  int i,j,k;
  float p,q,max;
  int posi, m;
  posi = 0;
  p = x_size*y_size;
  max = 0;
  for(i=0;i<256;i++) if(hist[i] > max) max = (float)hist[i];
  for(i=0;i<256;i++){
    q=(float)((float)hist[i]/p*100.0);
    m=sprintf(&buf[posi], "%3d:%5.1f%%|",i,q);
    posi += m;
    k = (int)((float)hist[i]/max*60.0);
    for(j=0;j<k;j++){
      m=sprintf(&buf[posi], "*");
      posi += m;
    }
    m = sprintf(&buf[posi], "\n");
    posi += m;
  }
}

/*--- thres_color --- R,G,B値による閾値処理 --------------------------------
  image_in_r:   入力R画像
  image_in_g:   入力G画像
  image_in_b:   入力B画像
  image_out_r:  出力R画像
  image_out_g:  出力G画像
  image_out_b:  出力B画像
  thdrl, thdrm: Rの閾値(min, max)
  thdgl, thdgm: Gの閾値(min, max)
  thdbl, thdbm: Bの閾値(min, max)
  ----------------------------------------------------------------------*/

void thresh_color(unsigned char image_in_r[MAX_IMAGESIZE][MAX_IMAGESIZE],
		  unsigned char image_in_g[MAX_IMAGESIZE][MAX_IMAGESIZE],
		  unsigned char image_in_b[MAX_IMAGESIZE][MAX_IMAGESIZE],
		  unsigned char image_out_r[MAX_IMAGESIZE][MAX_IMAGESIZE],
		  unsigned char image_out_g[MAX_IMAGESIZE][MAX_IMAGESIZE],
		  unsigned char image_out_b[MAX_IMAGESIZE][MAX_IMAGESIZE],
		  int thdrl, int thdrm, int thdgl, int thdgm, int thdbl, int thdbm)
{
  int i, j;
  for(i = 0; i < Y_SIZE; i++){
    for(j = 0; j < X_SIZE; j++){
      image_out_r[i][j] = image_in_r[i][j];
      image_out_g[i][j] = image_in_g[i][j];
      image_out_b[i][j] = image_in_b[i][j];
    }
  }
  for(i = 0; i < Y_SIZE; i++){
    for(j = 0; j < X_SIZE; j++){
      if(image_out_r[i][j] < thdrl)
	image_out_r[i][j] = image_out_g[i][j] = image_out_b[i][j] = 0;
      if(image_out_r[i][j] > thdrm)
	image_out_r[i][j] = image_out_g[i][j] = image_out_b[i][j] = 0;
      if(image_out_g[i][j] < thdgl)
	image_out_r[i][j] = image_out_g[i][j] = image_out_b[i][j] = 0;
      if(image_out_g[i][j] > thdgm)
	image_out_r[i][j] = image_out_g[i][j] = image_out_b[i][j] = 0;
      if(image_out_b[i][j] < thdbl)
	image_out_r[i][j] = image_out_g[i][j] = image_out_b[i][j] = 0;
      if(image_out_b[i][j] > thdbm)
	image_out_r[i][j] = image_out_g[i][j] = image_out_b[i][j] = 0;
    }
  }
}

void gradient(unsigned char image_in[MAX_IMAGESIZE][MAX_IMAGESIZE],
	      unsigned char image_out[MAX_IMAGESIZE][MAX_IMAGESIZE], float amp)
{
  static int cx[9] = {0, 0, 0,     /* オペレータの係数x(Roberts)  */
		      0, 1, -1,     /* 他のオペレータの時は        */
		      0, 0, 0};   /* 書き変えて下さい            */
  static int cy[9] = {0, 0, 0,     /* オペレータの係数y(Roberts)  */
		      0, 1, 0,     /* 他のオペレータの時は        */
		      0,-1, 0};    /* 書き変えて下さい            */
  int d[9];
  int i,j,dat;
  float xx,yy,zz;

  for(i=1;i<Y_SIZE-1;i++){
    for(j=1;j<X_SIZE-1;j++){
      d[0] = image_in[i-1][j-1];
      d[1] = image_in[i-1][j];
      d[2] = image_in[i-1][j+1];
      d[3] = image_in[i][j-1];
      d[4] = image_in[i][j];
      d[5] = image_in[i][j+1];
      d[6] = image_in[i+1][j-1];
      d[7] = image_in[i+1][j];
      d[8] = image_in[i+1][j+1];
      xx = (float)(cx[0]*d[0] + cx[1]*d[1] + cx[2]*d[2]
		   + cx[3]*d[3] + cx[4]*d[4] + cx[5]*d[5]
		   + cx[6]*d[6] + cx[7]*d[7] + cx[8]*d[8]);
      yy = (float)(cy[0]*d[0] + cy[1]*d[1] + cy[2]*d[2]
		   + cy[3]*d[3] + cy[4]*d[4] + cy[5]*d[5]
		   + cy[6]*d[6] + cy[7]*d[7] + cy[8]*d[8]);

      zz = (float)(amp*sqrt(xx*xx + yy*yy));
      dat = (int)zz;
      if(dat > 256) dat = 255;
     image_out[i][j] = (char)dat;
    }
  }
}

void threshold(unsigned char image_in[MAX_IMAGESIZE][MAX_IMAGESIZE],unsigned char image_out[MAX_IMAGESIZE][MAX_IMAGESIZE],int thres)
{ 
   int x,y;
  for(y=0;y<Y_SIZE;y++){
     for(x=0;x<X_SIZE;x++){
           if(image_in[y][x]>thres) image_out[y][x]=255;
                 else image_out[y][x]=0;
      }
   }
}

/*=============================================================================
	関数名  load_pgm_data
	用途    PGM画像の入力
	引数    char *file_name				: 入力画像ファイル名のポインタ
			unsigned char image_		: 入力画像格納配列のポインタ
			int x_size					: 画像横サイズのポインタ
			int y_size					: 画像横サイズのポインタ
	返り値  non
=============================================================================*/
void load_pgm_data(char *file_name,unsigned char image[MAX_IMAGESIZE][MAX_IMAGESIZE],int *x_size, int *y_size )
{
    char buffer[MAX_BUFFERSIZE];  /* データ読み込み用作業変数 */
    FILE *fp; /* ファイルポインタ */
    int max_gray; /* 最大階調値 */
    int x, y; /* ループ変数 */
    /* 入力ファイルのオープン */
	FILE_OPEN_ERROR(fopen( file_name, "rb" ));
    /* ファイルタイプ(=P5)の確認 */
    fgets( buffer, MAX_BUFFERSIZE, fp );
    if ( buffer[0] != 'P' || buffer[1] != '5' ){
        printf("ファイルのフォーマットが P5 とは異なります．¥n");
        exit(1);
    }
    /* x_size1, y_size1 の代入（#から始まるコメントは読み飛ばす） */
    *x_size = 0;
    *y_size = 0;
    while ( *x_size == 0 || *y_size == 0 ){
        fgets( buffer, MAX_BUFFERSIZE, fp );
        if ( buffer[0] != '#' ){
            sscanf( buffer, "%d %d", x_size, y_size );
        }
    }
    /* max_gray の代入（#から始まるコメントは読み飛ばす） */
    max_gray = 0;
    while ( max_gray == 0 ){
        fgets( buffer, INPUT_GRAY_SIZE, fp );
//		fgets( buffer, MAX_BUFFERSIZE, fp );
        if ( buffer[0] != '#' ){
            sscanf( buffer, "%d", &max_gray );
        }
    }
    /* パラメータの画面への表示 */
    if ( *x_size > MAX_IMAGESIZE || *y_size > MAX_IMAGESIZE ){
        printf("想定値 %d x %d を超えています．\n", 
            MAX_IMAGESIZE, MAX_IMAGESIZE);
        printf("もう少し小さな画像を使って下さい．\n");
        exit(1);
    }
    if ( max_gray != MAX_BRIGHTNESS ){
        printf("最大階調値が不適切です．¥n");
        exit(1);
    }
    /* 画像データを読み込んで画像用配列に代入する */
    for ( y = 0; y < *y_size; y ++ ){
        for ( x = 0; x < *x_size; x ++ ){
            image[y][x] = (unsigned char)fgetc( fp );
        }
    }
    fclose(fp);
}


/*=============================================================================
	関数名  save_pgm_data
	用途    PGM画像の保存
	引数    char *file_name				: 出力画像ファイル名のポインタ
			unsigned char image_		: 出力画像格納配列のポインタ
			int x_size					: 画像横サイズ
			int y_size					: 画像横サイズ
	返り値  non
=============================================================================*/
void save_pgm_data(char *file_name,unsigned char image[MAX_IMAGESIZE][MAX_IMAGESIZE],int x_size, int y_size)
/* image2[ ][ ], x_size2, y_size2 のデータを，それぞれ ppm 画像，*/
/* 横画素数，縦画素数としてファイルに保存する．                  */
{
    FILE *fp; /* ファイルポインタ */
    int x, y; /* ループ変数 */
    /* 出力ファイルのオープン */
	FILE_OPEN_ERROR(fopen(file_name, "wb"));
    /* ファイル識別子 "P5" を先頭に出力する */
    fputs( "P5\n", fp );
    /* # で始まるコメント行（省略可能） */
    fputs( "# Created by Image Processing\n", fp );
    /* 画像の横幅，縦幅の出力 */
    fprintf( fp, "%d %d\n", x_size, y_size );
    /* 最大階調値の出力 */
    fprintf( fp, "%d\n", MAX_BRIGHTNESS );

    for ( y = 0; y < y_size; y++)
        for ( x = 0; x < x_size; x++)
            fputc( image[y][x], fp );
    fclose(fp);
}
    
 /*--- laplacian --- 2次微分による輪郭抽出 -----------------------------
  image_in:   入力画像配列
  image_out:  出力画像配列
  amp:        出力画像の利得
  ---------------------------------------------------------------------*/

void laplacian_color(unsigned char image_in[3][MAX_IMAGESIZE][MAX_IMAGESIZE],
	       unsigned char image_out[3][MAX_IMAGESIZE][MAX_IMAGESIZE], float amp)
{
  static int c[9] = {-1,-1,-1    /*  オペレータの係数(laplacian)  */
		     -1, 8,-1    /*  他のオペレータ使用時は       */
		     -1,-1,-1};  /*  書き変えて下さい             */
  int d[9];
  int i,j,k,dat;
  float z, zz;
  for(k=0;k<3;k++){
   for(i=1;i<Y_SIZE-1;i++){
     for(j=1;j<X_SIZE-1;j++){
      d[0] = image_in[k][i-1][j-1];
      d[1] = image_in[k][i-1][j];
      d[2] = image_in[k][i-1][j+1];
      d[3] = image_in[k][i][j-1];
      d[4] = image_in[k][i][j];
      d[5] = image_in[k][i][j+1];
      d[6] = image_in[k][i+1][j-1];
      d[7] = image_in[k][i+1][j];
      d[8] = image_in[k][i+1][j+1];
      z=(float)(c[0]*d[0]+c[1]*d[1]+c[2]*d[2]
		+c[3]*d[3]+c[4]*d[4]+c[5]*d[5]
		+c[6]*d[6]+c[7]*d[7]+c[8]*d[8]);
      zz = amp*z;
      dat = (int)(zz);
      //if(dat<0) dat = -dat;
      if(dat>255) 
      image_out[0][i][j] = image_out[1][i][j] = image_out[2][i][j]=255;
    }
  }
 }
}
       
