// -*- C++ -*-
/*!
 * @file  Interface.cpp
 * @brief Console output component
 * $Date$
 *
 * $Id$
 */

#include "Interface.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>
#include <memory.h>
#include <math.h>

#include "shm_interface.h"

#define RGB               3		/* $B?'>pJs(B */
#define MAX_IMAGESIZE	1024	/* $BA[Dj$9$k=D!&2#$N:GBg2hAG?t(B */
#define MAX_BRIGHTNESS  255		/* $BA[Dj$9$k:GBg3,D4CM(B */

int shm_initialize(key_t shm_key, int shm_size,int *shrd_id,int *shm_ptr);
int shm_access(key_t shm_key, int shm_size, int *shrd_id, int *shm_ptr);

int shd_param_id;
struct SHM_PARAM *param_ptr;
double *data_ptr;
key_t key;

void save_ppm_data(char *file_name,unsigned char image[3][240][320],int x_size, int y_size); /*pgm$B%G!<%?$rJ]B8(B*/

short i, j, k;
unsigned char image_out_L[3][240][320];
unsigned char image_out_R[3][240][320];

// Module specification
// <rtc-template block="module_spec">
static const char* interface_spec[] =
  {
    "implementation_id", "Interface",
    "type_name",         "Interface",
    "description",       "Console output component",
    "version",           "1.0",
    "vendor",            "MyName",
    "category",          "example",
    "activity_type",     "DataFlowComponent",
    "max_instance",      "10",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables

    ""
  };
// </rtc-template>

Interface::Interface(RTC::Manager* manager)
  : RTC::DataFlowComponentBase(manager),
    // <rtc-template block="initializer">
    Cam_inIn("Cam_in", Cam_in),
    Laser_inIn("Laser_in", Laser_in),
    Cam_outOut("Cam_out", Cam_out),
    Laser_outOut("Laser_out", Laser_out),
    
    // </rtc-template>
	dummy(0)
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  registerInPort("Cam_in", Cam_inIn);
  registerInPort("Laser_in", Laser_inIn);
  
  // Set OutPort buffer
  registerOutPort("out", Cam_outOut);
  registerOutPort("out", Laser_outOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

}

Interface::~Interface()
{
}


RTC::ReturnCode_t Interface::onInitialize()
{
  // <rtc-template block="bind_config">
  // Bind variables and configuration variable

	// SHM$B$N5/F0(B
	system("./shmexe_interface &");
	sleep(1);

	//	GUI$B$N5/F0(B
	system("./GUI2 &");

	key = ftok(KEYFILE_PATH,PROJ_CHAR);
  if( key == -1 ) {
    perror("ftok()");
  }

  shm_access(key, sizeof(struct SHM_PARAM), &shd_param_id, (int *)&param_ptr);
  param_ptr->shd_param_id = shd_param_id;

  // </rtc-template>
  return RTC::RTC_OK;
}



/*
RTC::ReturnCode_t Interface::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Interface::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Interface::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Interface::onActivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Interface::onDeactivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t Interface::onExecute(RTC::UniqueId ec_id)
{
	//	$B%+%a%i5/F0(B
	if (param_ptr->cam == 1)
	{
		Cam_out.data = 1;
		Cam_outOut.write();

		//	$B2hA|<hF@BT$A(B
		while (1)
		{
			if (Cam_inIn.isNew())
			{
				Cam_inIn.read();
				for(i=0;i<3;i++){
					for(j=0;j<240;j++){
						for(k=0;k<320;k++){
							image_out_L[i][j][k] = Cam_in.data.gazou_L[i][j][k];
							image_out_R[i][j][k] = Cam_in.data.gazou_R[i][j][k];
	          }
	        }
	      }

				//$B2hA|=hM}8e$N2hA|$r=PNO(B
				save_ppm_data("output1.ppm", image_out_L, 320, 240);
				save_ppm_data("output2.ppm", image_out_R, 320, 240);

				param_ptr->cam = 2;
				break;
			}
		}
	}

	//	$B%l!<%65/F0(B
	if (param_ptr->las == 1)
	{
		Laser_out.data = 1;
		Laser_outOut.write();

		//	$B%l!<%6%G!<%?<hF@BT$A(B
		while (1)
		{
			if (Laser_inIn.isNew())
			{
				double x[720];
				double y[720];
				short i;
				FILE *fp;

				Laser_inIn.read();

				fp = fopen("scan.txt", "w");

				for (i=0;i<720;i++)
				{
					x[i] = Laser_in.data.scan_x[i];
					y[i] = Laser_in.data.scan_y[i];
					fprintf(fp, "%lf\t%lf\n", x[i], y[i]);
				}

				fclose(fp);
				
				param_ptr->las = 2;
				break;
			}
		}
	}
	
  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t Interface::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Interface::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Interface::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Interface::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Interface::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void InterfaceInit(RTC::Manager* manager)
  {
    RTC::Properties profile(interface_spec);
    manager->registerFactory(profile,
                             RTC::Create<Interface>,
                             RTC::Delete<Interface>);
  }
  
};

void save_ppm_data(char *file_name,unsigned char image[3][240][320],int x_size, int y_size)
/* image2[ ][ ], x_size2, y_size2 $B$N%G!<%?$r!$$=$l$>$l(B ppm $B2hA|!$(B*/
/* $B2#2hAG?t!$=D2hAG?t$H$7$F%U%!%$%k$KJ]B8$9$k!%(B                  */
{
    FILE *fp; /* $B%U%!%$%k%]%$%s%?(B */
    int x, y, k; /* $B%k!<%WJQ?t(B */
    /* $B=PNO%U%!%$%k$N%*!<%W%s(B */
    fp=fopen(file_name, "wb");
	
    /* $B%U%!%$%k<1JL;R(B "P6" $B$r@hF,$K=PNO$9$k(B */
    fputs( "P6\n", fp );
    /* # $B$G;O$^$k%3%a%s%H9T!J>JN,2DG=!K(B */
    //fputs( "# Created by Image Processing\n", fp );

    /* $B2hA|$N2#I}!$=DI}$N=PNO(B */
    fprintf( fp, "%d %d\n", x_size, y_size );

    /* $B:GBg3,D4CM$N=PNO(B */
    fprintf( fp, "%d\n", MAX_BRIGHTNESS );

    /* $B2hA|%G!<%?$N=PNO(B */
    for ( y = 0; y < 240/*y_size*/; y ++ ){
        for ( x = 0; x <320/* x_size*/; x ++ ){
		 	for ( k = 0; k < RGB; k ++ ){
            	fputc( image[k][y][x], fp );
                        }
          }
     }
	fclose(fp);
}

