#ifndef SCIP1_H
#define SCIP1_H
/*!
  @file
  @brief SCIP1 API
  @author aokaze (HARA Yoshitaka) <bluewind@roboken.esys.tsukuba.ac.jp>
  @author YoS (YOSHIDA Tomoaki) <yoshida@furo.org>
  @par    Publisher:
          HOKUYO AUTOMATIC CO.,LTD.
  @since  2005
  @date   2007/03/30
*/

#include "scipBase.h"

//! センサのバージョン情報表示[Vコマンド]
/*!
  @param aPort [in] センサとの通信ハンドル
  @retval 0 成功
  @retval -1 コマンドの送受信に失敗
  @retval -2 不良伝文応答（未定義コマンド）
  @retval other 失敗（応答ステータス）
*/
int scip1ShowVersion(tPort* aPort);

//! レーザ発光を制御[Lコマンド]
/*!
  @param aPort [in] センサとの通信ハンドル
  @param aState [in] パラメータ（0：レーザ消灯、1：レーザ点灯）
  @retval 0 成功
  @retval -1 コマンドの送受信に失敗
  @retval -2 不良伝文応答（未定義コマンド）
  @retval -3 パラメータ不良（0,1以外）
  @retval other 失敗（応答ステータス）
*/
int scip1SwitchLaser(tPort* aPort, int aState);

//! 通信速度を変更[Sコマンド]
/*!
  @param aPort [in] センサとの通信ハンドル
  @param aSpeed [in] 設定する通信速度 (bps)
  @retval 0 成功
  @retval -1 コマンドの送受信に失敗
  @retval -2 不良伝文応答（未定義コマンド）
  @retval -3 通信速度設定に失敗
  @retval other 失敗（応答ステータス）
*/
int scip1SetComSpeed(tPort* aPort, int aSpeed);

//! 測定済みスキャンデータを取得[Gコマンド]
/*!
  @param aPort [in] センサとの通信ハンドル
  @param aStartStep [in] スキャンの開始方向 (0-768)
  @param aEndStep [in] スキャンの終了方向 (0-768)
  @param aStepCluster [in] データ取得時にまとめる方向数 (0-99)
  @param aStepNum [out] スキャンデータのステップ数 (1-769)
  @return 取得したスキャンの距離データ配列の先頭アドレス
  malloc したメモリへのポインタなので、使用後に free する必要がある。
  スキャン取得に失敗したときは NULL を返す。
*/
int* scip1GetScan(tPort* aPort, int aStartStep, int aEndStep, int aStepCluster, int *aStepNum);

//! malloc したスキャンの距離データ配列を free
/*!
  @param aScan [in] aScan 破棄するスキャンの距離データ配列の先頭アドレス
  @retval EXIT_SUCCESS 成功
  @retval EXIT_FAILURE 失敗
 */
int scip1FreeScan(int *aScan);

#endif  // SCIP1_H
