#include <stdio.h>
#include "scipUtil.h"

int scipTryComm(tPort* aPort)
{
  char cmd[CMD_SIZE];
  char line[LINE_SIZE];
  int ret;

  flushSensorBuf(aPort->dev);
  fprintf(stdout,"Trying to communicate in SCIP2 ...\n");
  sprintf(cmd,"VV\n");
  ret=scipSendCmd(aPort,cmd,2);
  while(fgets(line,sizeof(line),aPort->dev)!=NULL)
    if(strcmp(line,LF_STRING)==0)
      break;
  if(ret==0){
    fprintf(stdout,"Communicating in SCIP2\n");
    aPort->protocol=2;
    aPort->state=STATE_IDLE;
    return 2;
  }

  flushSensorBuf(aPort->dev);
  return -1;
}

int scipTryComm2(tPort* aPort)
{
  char cmd[CMD_SIZE];
  char line[LINE_SIZE];
  int ret;

  flushSensorBuf(aPort->dev);
  fprintf(stdout,"Trying to communicate in SCIP1 ...\n");
  sprintf(cmd,"V\n");
  ret=scipSendCmd(aPort,cmd,1);
  while(fgets(line,sizeof(line),aPort->dev)!=NULL)
    if(strcmp(line,LF_STRING)==0)
      break;
  if(ret==0){
    fprintf(stdout,"Communicating in SCIP1\n");
    aPort->protocol=1;
    aPort->state=STATE_MEASURE_STORE;
    return 1;
  }

  flushSensorBuf(aPort->dev);
  fprintf(stdout,"Trying to communicate in SCIP2 ...\n");
  sprintf(cmd,"VV\n");
  ret=scipSendCmd(aPort,cmd,2);
  while(fgets(line,sizeof(line),aPort->dev)!=NULL)
    if(strcmp(line,LF_STRING)==0)
      break;
  if(ret==0){
    fprintf(stdout,"Communicating in SCIP2\n");
    aPort->protocol=2;
    aPort->state=STATE_IDLE;
    return 2;
  }

  flushSensorBuf(aPort->dev);
  return -1;
}

/*************long_laser************/
tPort* scipConnect(char *aDevName)
{
  tPort *port;
  int ret;

  fprintf(stdout,"Connecting at 19200 bps ...\n");	//OK
  port=openPort(aDevName,19200);
  ret=scipTryComm(port);	//	this file
  switch(ret){
    case 1:
    case 2:
      fprintf(stdout,"Connected at 19200 bps\n\n");
      return port;
    default:
      fprintf(stdout,"Could not connect at 19200 bps\n\n");
      break;
  }

  fprintf(stdout,"Connecting at 57600 bps ...\n");
  if(setupPort(fileno(port->dev),57600)!=EXIT_SUCCESS){
    perror("Could not set terminal parameters ");
    return NULL;
  }
  ret=scipTryComm(port);
  switch(ret){
    case 1:
    case 2:
      fprintf(stdout,"Connected at 57600 bps\n\n");
      return port;
    default:
      fprintf(stdout,"Could not connect at 57600 bps\n\n");
      break;
  }

  fprintf(stdout,"Connecting at 115200 bps ...\n");
  if(setupPort(fileno(port->dev),115200)!=EXIT_SUCCESS){
    perror("Could not set terminal parameters ");
    return NULL;
  }
  ret=scipTryComm(port);
  switch(ret){
    case 1:
    case 2:
      fprintf(stdout,"Connected at 115200 bps\n\n");
      return port;
    default:
      fprintf(stdout,"Could not connect at 115200 bps\n\n");
      break;
  }

  return NULL;
}


/**********short_laser***************/
tPort* scipConnect2(char *aDevName)
{
  tPort *port;
  int ret;

  fprintf(stdout,"Connecting at 19200 bps ...\n");
  port=openPort(aDevName,19200);
  ret=scipTryComm2(port);
  switch(ret){
    case 1:
    case 2:
      fprintf(stdout,"Connected at 19200 bps\n\n");
      return port;
    default:
      fprintf(stdout,"Could not connect at 19200 bps\n\n");
      break;
  }

  fprintf(stdout,"Connecting at 57600 bps ...\n");
  if(setupPort(fileno(port->dev),57600)!=EXIT_SUCCESS){
    perror("Could not set terminal parameters ");
    return NULL;
  }
  ret=scipTryComm2(port);
  switch(ret){
    case 1:
    case 2:
      fprintf(stdout,"Connected at 57600 bps\n\n");
      return port;
    default:
      fprintf(stdout,"Could not connect at 57600 bps\n\n");
      break;
  }

  fprintf(stdout,"Connecting at 115200 bps ...\n");
  if(setupPort(fileno(port->dev),115200)!=EXIT_SUCCESS){
    perror("Could not set terminal parameters ");
    return NULL;
  }
  ret=scipTryComm2(port);
  switch(ret){
    case 1:
    case 2:
      fprintf(stdout,"Connected at 115200 bps\n\n");
      return port;
    default:
      fprintf(stdout,"Could not connect at 115200 bps\n\n");
      break;
  }

  return NULL;
}

int id2step(int aStartStep, int aStepCluster, int aId)
{
  aStepCluster = aStepCluster==0 ? 1 : aStepCluster;
  return aStartStep + aId * aStepCluster;
}

double step2rad(int aStep)
{
  return (aStep-FRONT_STEP)*ANG_UNIT_RAD;
}
