#ifndef SCIP_UTIL_H
#define SCIP_UTIL_H
/*!
  @file
  @brief SCIP ユーザプログラミング補助 API
  @author aokaze (HARA Yoshitaka) <bluewind@roboken.esys.tsukuba.ac.jp>
  @author YoS (YOSHIDA Tomoaki) <yoshida@furo.org>
  @par    Publisher:
          HOKUYO AUTOMATIC CO.,LTD.
  @since  2006
  @date   2007/03/30
*/

#include "scipBase.h"
#include "scip1.h"
#include "scip2.h"

#define ANG_UNIT_RAD (2.0*M_PI / ANG_RESOLUTION)

//! SCIP1 および SCIP2 での通信を試みる（通信速度は変更しない）
/*!
  @param aPort [in] センサとの通信ハンドル
  @retval 1 SCIP1 での通信が確立
  @retval 2 SCIP2 での通信が確立
  @retval -1 SCIP1 および SCIP2 での通信に失敗
*/
int scipTryComm(tPort* aPort);
int scipTryComm2(tPort* aPort);
//! 指定されたシリアルポートに SCIP で接続する
/*!
  指定されたシリアルポートを開き、複数の通信速度において SCIP1 および SCIP2 での通信を試みてセンサと接続する。
  @param aDevName [in] シリアルポートのデバイスファイル名
  @return センサとの通信ハンドル
  失敗したときは NULL を返す
*/
tPort* scipConnect(char *aDevName);
tPort* scipConnect2(char *aDevName);
//! スキャンデータの配列番号を方向（ステップ番号）に変換
/*!
  @param aStartStep [in] スキャン開始方向
  @param aStepCluster [in] データ取得時にまとめた方向数 (1-99)
  @param aId [in] 配列番号
  @return 方向（ステップ番号）
*/
int id2step(int aStartStep, int aStepCluster, int aId);

//! 方向（ステップ番号）を radian 角度に変換
/*!
  @param aStep [in] 方向（ステップ番号）
  @return radian 角度
*/
double step2rad(int aStep);

#endif  // SCIP_UTIL_H
